#!/bin/env python

import sys
import json
from os import path
from string import Template
import requests

LIFF_BASE_URL = "https://api.line.me/liff/v1/apps"

KEY_LIFF_ID = "liffId"
KEY_VIEW = "view"
KEY_TYPE = "type"
KEY_URL = "url"
KEY_APPS = "apps"
KEY_MESSAGE = "message"

APP_INFO_TEMPLATE = """
  id: $liff_id,
  viewType: $view_type,
  url: $url"""


class LiffClient:

    home = path.expanduser("~")
    setting_file = path.join(home, ".liff")
    tokens = {}
    default = ""

    def __init__(self):
        try:
            self._load_settings()
            self.default_token = self.tokens[self.default]
        except Exception as error:
            self.default_token = ""

    def _load_settings(self):
        with open(self.setting_file, "r") as json_file:
            settings = json.load(json_file)
            self.tokens = settings["tokens"]
            self.default = settings["default"]

    def add_access_token(self, name, token):
        if name in self.tokens:
            print("[WARNING] the name is existed, overwrite the token")

        self.tokens[name] = token
        self.default = name
        self._serialize()

    def delete_access_token(self, name):
        key_to_delete = ""
        #       cannot change the size of dict during iteration
        for key in self.tokens:
            if name == key:
                print("delete the token of " + name)
                key_to_delete = key

        if key_to_delete:
            del self.tokens[key_to_delete]
            if self.default == name:
                all_keys = list(self.tokens.keys())
                if len(all_keys) > 0:
                    self.default = all_keys[-1]
                    self.default_token = self.tokens[self.default]

            self._serialize()
        else:
            print("No token found for " + name)

    def _serialize(self):
        json_data = {"default": self.default, "tokens": self.tokens}
        with open(self.setting_file, "w") as json_file:
            json.dump(json_data, json_file, indent=4)

    def set_default_token(self, name):
        if name in self.tokens:
            self.default = name
            self._serialize()
        else:
            print("No token for " + name)

    def list_all_tokens(self):
        if len(self.tokens) == 0:
            print("No tokens")
            return

        for key in self.tokens:
            output = "[{0}]: {1}".format(key, self.tokens[key])
            if key == self.default:
                output = "* " + output
            else:
                output = "  " + output

            print(output)

    def create_app(self, url, size):
        if not self._verify_view_size(size):
            print("The value of size is incorrect, it must be 'full', 'tall' or 'compact'")
            return

        def success_handler(json):
            print("Create LIFF app successfully, id: " + json[KEY_LIFF_ID])

        def error_handler(json):
            print("Could not create LIFF app. Reason: " + json[KEY_MESSAGE])
            sys.exit(1)

        payload = {KEY_VIEW: {KEY_TYPE: size, KEY_URL: url}}
        response = requests.post(LIFF_BASE_URL, headers=self._default_headers(), json=payload)
        self._handle_response(response, success_handler, error_handler)

    def delete_app(self, liff_id):

        def success_handler(json):
            print("Delete LIFF app successfully")

        def error_handler(json):
            print("Could not delete LIFF app. Reason: " + json[KEY_MESSAGE])
            sys.exit(1)

        response = requests.delete(LIFF_BASE_URL + "/{0}".format(liff_id), headers=self._default_headers())
        self._handle_response(response, success_handler, error_handler)

    def update_app(self, liff_id, properties):
        if not properties:
            print("The new value is empty, skip")
            return

        def success_handler(json):
            print("Update successfully")

        def error_handler(json):
            print("Cannot update LIFF app. Reason: " + json[KEY_MESSAGE])
            sys.exit(1)

        payload = self._generate_json_for_update(properties)
        response = requests.put(
            LIFF_BASE_URL + "/{0}/view".format(liff_id),
            headers=self._default_headers(),
            json=payload
        )
        self._handle_response(response, success_handler, error_handler)

    def _generate_json_for_update(self, properties):
        new_properties = properties.split(",")
        new_values = {}
        for property in new_properties:
            key_value = property.strip().split(":", maxsplit=1)
            if len(key_value) != 2:
                print("Incorrect value format for updating liff application")
                sys.exit(1)
            key = key_value[0].strip()
            value = key_value[1].strip()
            if self._verify_key_and_value(key, value):
                new_values[key] = value

        return new_values

    def _verify_key_and_value(self, key, value):
        if not key == KEY_TYPE and not key == KEY_URL:
            return False
        if key == KEY_TYPE and not self._verify_view_size(value):
            print("The value of size is incorrect, it must be 'full', 'tall' or 'compact'")
            sys.exit(1)

        return True

    def list_all_apps(self):
        if not self.default_token:
            print("Please set access token first, run liff token add <name> <token>")
            sys.exit(1)

        response_template = Template(APP_INFO_TEMPLATE)
        response = requests.get(LIFF_BASE_URL, headers=self._default_headers())

        def success_handler(json):
            if KEY_APPS not in json:
                print("No LIFF application now, use 'liff create' to create a new LIFF application.")

            apps = json[KEY_APPS]
            for app in apps:
                print(response_template.substitute(
                    liff_id=app[KEY_LIFF_ID], view_type=app[KEY_VIEW][KEY_TYPE], url=app[KEY_VIEW][KEY_URL]))

        def error_handler(json):
            print("Could not list LIFF app. Reason: " + json[KEY_MESSAGE])
            sys.exit(1)

        self._handle_response(response, success_handler, error_handler)

    def _verify_view_size(self, size):
        return size == "full" or size == "compact" or size == "tall"

    def _default_headers(self):
        return {"Authorization": "Bearer " + self.default_token,
                "Content-Type": "application/json"}

    def _handle_response(self, response, success_handler, error_handler):
        try:
            json_response = json.loads(response.text)
        except json.JSONDecodeError:
            json_response = ""
        if response.status_code != 200:
            error_handler(json_response)
        else:
            success_handler(json_response)



import sys
from string import Template
from .liffclient import LiffClient


TOKEN_USAGE = """  token add <name> <token>                        Add new access token, the new token will be the default token
  token[t] delete <name>                          Delete the access token. If it is default token, the last added token would become default token
  token[t] set-default <name>                     Set the default token
  token[t] list                                   List all tokens
"""

LIFF_APP_USAGE = """  create[c] <url> <type>[compact, tall, full]     Create a new LIFF application.
  list[l]                                         List all LIFF applications
  delete[d] <id>                                  Delete a LIFF application.
  update[u] <id> <new_values>                     Update the LIFF application with new values. The value format is PROPERTY:VALUE and use the comma to seperate each value
  Example: 
      liff update 1234567 url:https://test.com,type:full
"""

ALL_USAGE = Template("""usage: liff <command> [<args>]

Token management commands:
$token_usage
App management commands
$liff_app_usage
""").substitute(token_usage=TOKEN_USAGE, liff_app_usage=LIFF_APP_USAGE)


def run_token_command(client, subcommand, args):
    if (subcommand == "add" or subcommand == "a") and len(args) == 2:
        client.add_access_token(args[0], args[1])
    elif (subcommand == "delete" or subcommand == "d") and len(args) == 1:
        client.delete_access_token(args[0])
    elif subcommand == "set-default" and len(args) == 1:
        client.set_default_token(args[0])
    elif (subcommand == "list" or subcommand == "l") and len(args) == 0:
        client.list_all_tokens()
    else:
        print("usage:\n" + TOKEN_USAGE)


def run_command(command, args):
    client = LiffClient()
    if command == "token" or command == "t":
        if len(args) < 1:
            print("usage:\n" + TOKEN_USAGE)
            return
        subcommand = args[0]
        try:
            args = args[1:]
        except IndexError:
            args = []

        run_token_command(client, subcommand, args)

    elif command == "list" or command == "l":
        client.list_all_apps()

    elif command == "create" or command == "c":
        if len(args) < 2:
            print("usage:\n" + LIFF_APP_USAGE)
            return

        client.create_app(url=args[0], size=args[1])

    elif command == "delete" or command == "d":
        if len(args) < 1:
            print("usage:\n" + LIFF_APP_USAGE)
            return
        client.delete_app(args[0])

    elif command == "update" or command == "u":
        if len(args) < 2:
            print("usage:\n" + LIFF_APP_USAGE)
            return

        client.update_app(liff_id=args[0], properties=args[1])
    else:
        print(ALL_USAGE)


def main():
    if len(sys.argv) < 2:
        print(ALL_USAGE)
        sys.exit(1)

    run_command(sys.argv[1], sys.argv[2:])



#!/usr/bin/env python
from setuptools import setup, find_packages
VERSION = "1.0.1"


setup_info = dict(
    # Metadata
    name="pyliff",
    version=VERSION,
    author="Freddie Wang",
    author_email="wangyung@gmail.com",
    url="https://github.com/wangyung/pyliff",
    download_url="",
    description="pyliff is a CLI tool to let developers to setup LIFF (LINE Frontend Framework) easily",
    license="BSD",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Build Tools",
    ],

    keywords="liff",

    # Package info
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    include_package_data=True,
    install_requires=[
        "requests",
    ],
    scripts=["bin/liff"],

    # Add _ prefix to the names of temporary build dirs
    options={
        "build": {"build_base": "_build"},
    },
    zip_safe=True,
)

setup(**setup_info)
